In this README file, we walk you through the dataset and scripts in order to reproduce our experiment. Note that some paths to access the projects are outdated and will need to be adjusted if running the code. 

# js_projects.zip

Contains our dataset of 100 projects used in this study. Build commands are available under the "scripts" section of the package.json of each project. The source map output is already configured (in webpack.config.js or rollup.js)

# Requirements 

pip install -r requirements.txt

# scripts

This folder contains the scripts necessary to collect the data and compute our results.

**scripts > get_bundled_version.py**

Parses the package.json file in every node_module subfolder to get the version of production dependencies installed in each project

**scripts > get_map_files.py**

Walks the directory containing the cloned JavaScript projects to locate and retrieve the path of source map files (i.e., files with .map extensions) for each project. 

**scripts > get_vuln_type.py**

Parses the title of every vulnerability to extract its type. 

**scripts > git_diff_analysis.py**

Preliminary analysis of maintenance tasks. Determines the type of action performed (addition, deletion, update, downgrade) on a dependency based on the git diff of the package.json file. 

**scripts > git_diff_parser.py**

Parses the git diff output obtained from the pydriller dictionary output and writes it in a readable csv format. 

**scripts > make_figure.py**

Generates figures from analysis. 

**scripts > package_json_hist.py**

Uses pydriller to mine the projects' package.json commits. Outputs the git diff as a dictionary. 

**scripts > parse_audit.py**

Parses the npm audit json output of each project. Extracts the package name, version, cve, GitHub Advisory Database url, and other meta. 

**scripts > parse_npm_ls.py**

Goes through every project and recusively gets the version of each installed package. Creates master file of all packages resulting from npm ls command for all projects. 

**scripts > parse_sourcemaps.py**

This script was modified from the original source https://github.com/unisil/source-map-parser. Extracts source map files "source contents" to get the production dependencies bundled by webpack or rollup. 

**scripts > resolve_missing_peer.py**

Initially, this script was in an attempt to automatically install missing peer dependencies. Due to too many errors, it ended up being used to report on how many projects have missing peer dependencies. 

**scripts > rq1_rq3_analysis.py**

Data manipulation and computing results for RQ1 and RQ3.

**scripts > rq2_analysis.py**

Data manipulation and computing results for RQ2.

**scripts > repo_mining**

This folder contains the scripts we wrote and used to collect and filter the projects of our dataset. The script **builder.py** was initially created to automatically build every project. Due to too many build errors, we opted for manually building each project. 

The files created by **filter_repos.py** are not part of this set but can be found in the GitHub history. Those files were archived since we only used the filtered projects in our study. Having the iterative "filtered" files was noisy to the repository. 

# data

This folder contains the data generated by our scripts. 

**data > bundled_deps_version.csv**

Contains all production dependencies and their version. 

**data > commits.csv**

Contains the package.json commits data. 

**data > dataset.csv**

Contains data on the projects of the dataset: repository name, module bundler used, sideEffects enabled, and for each project, a list of the paths to map files. 

**data > deps_and_diffs.csv**

Contains the package name and version associated to a line change in the package.json commit data. 

**data > git_diff.csv**

Unprocessed version of deps_and_diffs.csv --> the insertions, deletions, and line number is given in a dictionary form with list of tuples as values. The script git_diff_parser.py was used to "explode" the dictionary values (in deps_and_diffs.csv).

**data > npm_audit_attack_types.csv**

Output of processing the json output of npm audit and the type of each vulnerability.

**data > npm_audit.csv**

Processed data from json output of npm audit.

**data > npm_ls_dependencies.csv**

Processed output from dependency tree of each project (npm ls). Contains all installed dependencies in each project.

**data > parsed_git_diff.csv**

Contains the change (insertions and deletions) for each line of the package.json file of every project. 

**data > per_project.csv**

Analysis of the dependency characteristics per project. 

**data > projects_dep_configs.csv**

Analysis of how projects configured their dependencies in the package.json file (Node convention, all in devDependencies, or all in dependencies).

**data > reports_per_project.csv**

Number of vulnerability report emitted for each project. 

**data > repos.csv**

Contains data on the projects of the dataset: repository name, module bundler used, sideEffects enabled, without the list of the paths to map files. 

**data > unique_deps_rq2.csv**

Data on how often an installed package is used in production versus not used in production. 


