# %%
import pandas as pd
import ast

"""
The git diff is given in the following format: 
{'added': [(59, 'dep1_v2'), (60, 'dep2_v2')], 'deleted': [(59, 'dep1_v1'), (60, 'dep2_v1')]}

We want to explode this so that we have a column for the line number, the added element, and the deleted element:
59, dep1_v2, dep1_v1. 
"""

diffs = pd.read_csv('../data/git_diff.csv')

def explode_records(df):
    new_records = []
    def map_dict_to_row(repo_name, value, change_type, commit_msg, diff_dict):
        temp = {}
        for number, added in diff_dict["added"]:
            temp[number] = {"repo_name": repo_name, "hash": value, "change_type": change_type, "commit_msg": commit_msg,  "line_number": number, "added": added}
        for number, deleted in diff_dict["deleted"]:
            if number in temp:
                temp[number] = {**temp[number], "deleted": deleted}
            else:
                temp[number] = {"repo_name": repo_name, "hash": value, "change_type": change_type, "commit_msg": commit_msg, "line_number": number, "deleted": deleted}
        new_records.extend(list(temp.values()))

    df.apply(lambda row: map_dict_to_row(row.repo_name, row.hash, row.change_type, row.msg, ast.literal_eval(row.diff_parsed)), axis=1) 
    return pd.DataFrame(new_records)

df_final = explode_records(diffs)
df_final.to_csv('../data/parsed_git_diff.csv', mode='w', index=False)

# %%
#dummy example to see if it works
import pandas as pd
import ast
diffs = ["{'added': [(59, 'dep1_v2'), (60, 'dep2_v2')], 'deleted': [(64, 'dep2_v1')]}",
         "{'added': [(61, 'dep3_v2')], 'deleted': [(61, 'dep3_v1')]}"]

df = pd.DataFrame({"col1": ["value1", "value2"], "col2": diffs})

def explode_records(df):
    new_records = []
    def map_dict_to_row(value, col2_dict):
        temp = {}
        for number, added in col2_dict["added"]:
            temp[number] = {"value": value, "number": number, "added": added}
        for number, deleted in col2_dict["deleted"]:
            if number in temp:
                temp[number] = {**temp[number], "deleted": deleted}
            else:
                temp[number] = {"hash": value, "deleted": deleted}
        new_records.extend(list(temp.values()))

    df.apply(lambda row: map_dict_to_row(row.hash, ast.literal_eval(row.diff_parsed)), axis=1)  # assumes col2 is a dict
    return pd.DataFrame(new_records)

df2 = explode_records(df)
# %%
