
# %%
from pydriller import Repository
from glob import glob
import csv

"""
We get the history of the projects package.json. 

Because we are interested in if dependencies are updated, downgraded, or removed, 
the most important information to us is:
commit hash, commit data, insertions, deletions
"""

#traverse_commits returns a generator, not list
#https://pydriller.readthedocs.io/en/latest/commit.html 
#https://pydriller.readthedocs.io/en/latest/repository.html#filtering-commits 

commits_out = open('data/commits.csv', 'w')
commits_csv_out = csv.DictWriter(commits_out, ['repo_name', 'hash', 'msg', 'author', 'committer', 
'author_date', 'author_timezone', 'committer_date', 'committer_timezone', 'branches', 
'in_main_branch', 'merge', 'parents', 'deletions', 
'insertions', 'lines', 'files'])
commits_csv_out.writeheader()

#mine gitdiff 
#https://pydriller.readthedocs.io/en/latest/modifiedfile.html
git_diff_out = open('data/git_diff.csv', 'w')
git_diff_csv_out = csv.DictWriter(git_diff_out, ['repo_name', 'hash', 'filename', 'change_type', 'diff_parsed', 'added_lines', 'deleted_lines'])
git_diff_csv_out.writeheader()

#Local repos
PATH = '../../../../Volumes/WINDOWS/cloned_js_projects/'
repo_urls = [s.rstrip('/') for s in glob(PATH + '*/')]

#Remote repos
"""
repositories = pd.read_csv('data/dataset.csv')
repo_names = repositories['full_repo_name']
repo_urls = []
for repo in repo_names: 
    name = 'https://github.com/' + repo + '.git'
    repo_urls.append(name)
"""

commits = Repository(repo_urls, filepath='package.json').traverse_commits()

for commit in commits:
    repo_name = commit.project_name
    hash = commit.hash
    msg = commit.msg
    author = commit.author.name
    committer = commit.committer.name
    author_date = commit.author_date
    author_timezone = commit.author_timezone
    committer_date = commit.committer_date
    committer_timezone = commit.committer_timezone
    branches = commit.branches
    in_main_branch = commit.in_main_branch
    merge = commit.merge
    parents = commit.parents
    deletions = commit.deletions
    insertions = commit.insertions
    lines = commit.lines
    files = commit.files
    
    msg = msg.replace('\n', ' ').replace('\r', '')

    commits_csv_out.writerow({'repo_name':repo_name, 'hash':hash, 'msg':msg, 'author':author, 'committer':committer, 
        'author_date':author_date, 'author_timezone':author_timezone, 'committer_date':committer_date, 'committer_timezone':committer_timezone, 'branches':branches, 
        'in_main_branch':in_main_branch, 'merge':merge, 'parents':parents, 'deletions':deletions, 
        'insertions':insertions, 'lines':lines, 'files':files})
    
    for file in commit.modified_files:
        if file.filename == 'package.json':
            filename = file.filename
            change_type = file.change_type
            diff_parsed = file.diff_parsed
            added_lines = file.added_lines
            deleted_lines = file.deleted_lines
  
            git_diff_csv_out.writerow({'repo_name': repo_name, 'hash': hash, 'filename': filename, 'change_type': change_type, 
                'diff_parsed': diff_parsed, 'added_lines': added_lines, 'deleted_lines': deleted_lines})
        else:
            pass
commits_out.close()
git_diff_out.close()
