# %%
import sys
import json
import pandas as pd
from glob import glob

"""
We build on the existing script https://github.com/nVisium/npm-deps-parser

This script parses, summerizes, and prints npm audit json output to markdown. 

We are specifically interested in the parsing and summary functions. Thus, we take the parsing and summary 
output and write it to a csv file. We are not using the markdown output. 
"""
# %%
def get_root_deps(paths):
    deps = []
    depths = []
    for d in paths:
        libs = d.split(">")
        root = libs[0] 
        depths.append(len(libs))
        if root not in deps:
            deps.append(root)
    return deps, str(max(depths))

def parse(repo_name, stdin=False, file_path="", cve_only=False, show_depth=False):
    if stdin:
        npm_json = json.load(sys.stdin)
        process_json(npm_json, repo_name, cve_only=cve_only, show_depth=show_depth)
    elif file_path is not "":
        with open(file_path) as json_file:
            npm_json = json.load(json_file)
        process_json(npm_json, repo_name, cve_only=cve_only, show_depth=show_depth)
    else:
        print("[-] You must specify a path or pass json via stdin using -i")

def process_json(npm_json, repo_name, cve_only=False, show_depth=False):
    cols = []
    if show_depth:
        cols = ['repo_name','cve', 'dependency_name', 'version', 'dependency_of', 'severity', 'depth', 'title', 'npm_bundled', 'dev', 'optional', 'info']
    else:
        cols = ['repo_name', 'cve', 'dependency_name', 'version', 'dependency_of', 'severity', 'title', 'npm_bundled', 'dev', 'optional', 'info']
    df = pd.DataFrame(columns=cols)
    for adv in npm_json["advisories"]:
        vuln = npm_json["advisories"][adv]
        cves = vuln["cves"]
        paths = vuln["findings"][0]["paths"]
        version = vuln['findings'][0]['version']
        deps,depth = get_root_deps(paths)
        deps_str = ", ".join(deps)
        severity = vuln['severity']
        npm_bundled = [v["resolves"][0]["bundled"] for v in npm_json["actions"] if vuln["module_name"] in v["resolves"][0]["path"].split(">") or vuln["module_name"] == v["module"]]
        dev = [v["resolves"][0]["dev"] for v in npm_json["actions"] if vuln["module_name"] in v["resolves"][0]["path"].split(">") or vuln["module_name"] == v["module"]]
        optional = [v["resolves"][0]["optional"] for v in npm_json["actions"] if vuln["module_name"] in v["resolves"][0]["path"].split(">") or vuln["module_name"] == v["module"]]

        if len(dev) == 0:
            dev = '?' 
        if len(optional) == 0:
            optional = '?'
        if len(npm_bundled) == 0:
            npm_bundled = '?'

        if len(cves) > 0:
            for cve in cves:
                if show_depth:
                    df = df.append({'repo_name': repo_name, 'cve': cve, 'dependency_name': vuln['module_name'], 'version': version,
                        'dependency_of': deps_str, 'severity': severity, 'depth': depth, 'title': vuln['title'], 'npm_bundled': npm_bundled[0], 
                        'dev': dev[0], 'optional': optional[0], 'info': vuln['url']}, ignore_index=True)

                else:
                    df = df.append({'repo_name': repo_name, 'cve': cve, 'dependency_name': vuln['module_name'], 'version': version,
                        'dependency_of': deps_str, 'severity': severity, 'title': vuln['title'], 'npm_bundled': npm_bundled[0],
                        'dev': dev[0], 'optional': optional[0], 'info': vuln['url']}, ignore_index=True)
            continue
        if not cve_only and len(cves) is 0:
            if show_depth:
                df = df.append({'repo_name': repo_name, 'cve': 'N/A', 'dependency_name': vuln['module_name'], 'version': version,
                        'dependency_of': deps_str, 'severity': severity, 'depth': depth, 'title': vuln['title'], 'npm_bundled': npm_bundled[0],
                        'dev': dev[0], 'optional': optional[0], 'info': vuln['url']}, ignore_index=True)

            else:
                df = df.append({'repo_name': repo_name, 'cve': 'N/A', 'dependency_name': vuln['module_name'], 'version': version,
                        'dependency_of': deps_str, 'severity': severity, 'title': vuln['title'], 'npm_bundled': npm_bundled[0],
                        'dev': dev[0], 'optional': optional[0], 'info': vuln['url']}, ignore_index=True)
        
        df.to_csv('../data/npm_audit.csv', mode='a', index=False, header=False)

def main():
    PATH = '../../../../Volumes/WINDOWS/cloned_js_projects/'
    repos = [s.rstrip('/') for s in glob(PATH + '*/')]

    for repo in repos:
        repo_name = repo.split('/')[6]
        npm_audit = repo + '/npm_audit.json'
        try:
            parse(repo_name, stdin=False, file_path=npm_audit, cve_only=False, show_depth=True)
        except:
            print('could not parse ' + repo)
            pass

if __name__ == "__main__":
    main()

# %%
