# %%
import json
import pandas as pd
from collections import namedtuple
import csv

"""
Goes through every repo and recusively gets the version of each package.
Creates master file of all packages resulting from npm ls command for all repo. 
"""

# %%
def recursive_search(root_pkg, path=tuple()):
    Package = namedtuple("Package", ["name", "version", "path"])
    for pkg_name, pkg in root_pkg.get("dependencies", {}).items():
        if "version" not in pkg:
            pkg['version'] = None
            print(f"Missing version for {pkg_name}" + ' ' + str(pkg['version']))
        yield Package(pkg_name, pkg["version"], path + (pkg_name,))
        yield from recursive_search(pkg, path + (pkg_name,))


df = pd.read_csv('../data/dataset.csv')
file_out = open('../data/npm_ls_dependencies.csv', 'w')
csv_out = csv.DictWriter(file_out, ['repo_name', 'dependency_name', 'version', 'path', 'depth', 'is_dev', 'is_runtime'])
csv_out.writeheader()
repos = df['repo_path'].unique()

for repo in repos:
    repo_name = str(repo).split('/')[-1]
    dependency_name = ''
    version = '' 
    path = []
    depth = 0
    print(repo_name)

    with open(repo + '/package.json', 'r') as f2:
        package_content = json.load(f2)
        dependencies = set(package_content.get('dependencies', {}).keys())
        dev_dependencies = set(package_content.get('devDependencies', {}).keys())
        dev_dependencies = dev_dependencies - dependencies #if dev and runtime --> runtime (edge case)

    with open(repo + '/npm_ls.json', 'r') as f:
        npm_content = json.load(f)
        for dep in recursive_search(npm_content):
            dependency_name = dep.name
            root = dep.path[0]
            version = dep.version   
            path = dep.path
            depth = len(dep.path)
            is_dev = root in dev_dependencies
            is_runtime = root in dependencies

            #csv lib --> write directly 
            csv_out.writerow({'repo_name': repo_name, 'dependency_name': dependency_name, 'version': version,
                                    'path': path, 'depth': depth, 'is_dev': is_dev, 'is_runtime': is_runtime})
        
file_out.close()


# %%
