import pandas as pd
from github import Github
import requests
import logging

"""
Simple script to clone projects locally. 
Before cloning, we look if the project has a package.json file since it is
the main component we are studying (dependencies). We also look if 
"sideEffects: false" since this indicates that a project has tree shaking (dead code elimination for dependencies)
"""

logging.basicConfig(level=logging.INFO,
                    format='%(asctime)s - %(levelname)s - %(message)s',
                    filename='build_logs',
                    filemode='w')
console = logging.StreamHandler()
console.setLevel(logging.INFO)
formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
console.setFormatter(formatter)
logging.getLogger('').addHandler(console)

token = ''
g = Github(token)

column_names = ['repo_name', 'stars', 'created_at', 'last_modified', 'size', 'nb_contributors', 'has_dependabot', 'webpack', 'rollup', 'sideEffects_value']
df = pd.DataFrame(columns = column_names)

repositories = pd.read_csv('../dataset.csv')
repo_names = repositories['full_repo_name']

for name in repo_names:
    logging.info('Checking repo ' + name)
    owner = name.split('/')[0]
    repository = name.split('/')[1]
    branch = repositories.loc[repositories['full_repo_name'] == name, 'default_branch'].values[0]
    url = f"https://raw.githubusercontent.com/{owner}/{repository}/{branch}/package.json"
    res = requests.get(url)
    try:
        content = res.json()
        if res.status_code == requests.codes.ok:
            logging.info('Status OK')
            if 'sideEffects' in content:
                logging.info('sideEffects found in repo ' + name)
                repo = g.get_repo(name)
                add_to_set = True
                contributors = repo.get_contributors()
                dependabot = False
                webpack = False
                rollup = False
                sideEffects = content['sideEffects']

                if 'dependencies' in content: 
                    if 'webpack' in content['dependencies']: 
                        logging.info('webpack found in repo lodash')
                        webpack = True
                    if 'rollup' in content['dependencies']: 
                        logging.info('rollup found in repo lodash')
                        rollup = True
                if 'devDependencies' in content:
                    if'webpack' in content['devDependencies']:
                        logging.info('webpack found in repo lodash')
                        webpack = True
                    if 'rollup' in content['devDependencies']:
                        logging.info('rollup found in repo lodash')
                        rollup = True

                for contributor in contributors:
                    if 'dependabot[bot]' == contributor.login:
                        dependabot = True

                repo_name = repo.full_name
                stars = repo.stargazers_count
                created_at = repo.created_at
                last_modified = repo.last_modified
                size = repo.size
                has_dependabot = dependabot
                nb_contributors = len(contributors)

                df = df.append({'repo_name': repo_name, 'stars': stars, 'created_at': created_at, 'last_modified': last_modified, 
                    'size': size, 'nb_contributors': nb_contributors, 'has_dependabot': has_dependabot, 'webpack': webpack, 'rollup': rollup, 'sideEffects_value': sideEffects}, ignore_index=True)
            
                df.to_csv('../repos.csv', index=False, mode='w', header=True)
            else:
                logging.info('sideEffects not found in repo ' + name)
        else:
            logging.info('Content not found, moving on to next repo: ' + name)
    except ValueError:
        logging.info('Could not decode JSON for repo ' + name)


    



    