# %%
from collections import namedtuple
import pandas as pd
import json

"""
We use the same method as in parse_npm_ls.py to go through the dependency tree 
and look for specific tags that indicate a peer dependency is missing. 

However, we could not automatically resolve missing peer dependencies, so we used
this script to report on the projects that do have missing peer dependencies. 
"""

def recursive_search(root_pkg, path=tuple(),):
    Package = namedtuple("Package", ["name", "install", "path"])
    for pkg_name, pkg in root_pkg.get("dependencies", {}).items():
        if "required" in pkg:
            pkg['install'] = pkg['required']['_id']
            yield Package(pkg_name, pkg["install"], path + (pkg_name,))
            yield from recursive_search(pkg, path + (pkg_name,))

df = pd.read_csv('../data/dataset.csv')
repos = df['repo_path'].unique()
repos_missing = []
for repo in repos:
    print('REPO: ' + repo)
    with open(repo + '/npm_ls.json', 'r') as f:
        npm_content = json.load(f)
        deps = []
        for dep in recursive_search(npm_content):
            deps.append(dep.install)
        if len(deps) != 0:
            repos_missing.append(repo)
            """for dep in deps:
                to_install += dep + ' '
            try:
                subprocess.check_call('npm install ' + to_install, cwd=repo, shell=True)
            except:
                print('Could not install peer deps for ' + repo)
                pass"""
print('done')

            
# %%
