# %%
import pandas as pd

##### Project level analysis #####

# projects that ship at least one dependency to production bundle
# %%
deps = pd.read_csv('../data/npm_ls_dependencies.csv')
deps = deps.groupby(["repo_name", "dependency_name", "version", "is_dev"]).agg(
        {"depth": "min", "path": "count"}).reset_index()

bundled_df = pd.read_csv('../data/bundled_deps_version.csv')

merged = deps.merge(bundled_df, on=['repo_name', 'dependency_name', 'version'])
merged = deps.merge(bundled_df[pd.notnull(bundled_df.is_bundled)], how='left', on=['repo_name', 'dependency_name', 'version'])
merged.fillna(False, inplace=True)
merged = merged.drop_duplicates()

bundled_projects = merged[merged['is_bundled'] == True]
bundled_projects = merged[merged['repo_name'].isin(bundled_projects['repo_name'])]

dep_count_per_project = bundled_projects.groupby(['repo_name']).size().to_frame('count').reset_index()
bundled_dep_per_project = bundled_projects[bundled_projects['is_bundled'] == True]
bundled_dep_count_per_project = bundled_dep_per_project.groupby(['repo_name']).size().to_frame('count').reset_index()

unbundled_dep_per_project = bundled_projects[bundled_projects['is_bundled'] == False]
unbundled_dep_count_per_project = unbundled_dep_per_project.groupby(['repo_name']).size().to_frame('count').reset_index()

depth_per_project = bundled_projects.groupby(['repo_name']).agg({'depth': 'max'})

## not per project, but per "dataset"
avg_bundled_projects_depth = bundled_projects['depth'].mean()
median_bundled_projects_depth = bundled_projects['depth'].median()
min_bundled_projects_depth = bundled_projects['depth'].min()
max_bundled_projects_depth = bundled_projects['depth'].max()

# projects that ship nothing
#%%
deps = pd.read_csv('../data/npm_ls_dependencies.csv')
deps = deps.groupby(["repo_name", "dependency_name", "version", "is_dev"]).agg(
        {"depth": "min", "path": "count"}).reset_index()

bundled_df = pd.read_csv('../data/bundled_deps_version.csv')

merged = deps.merge(bundled_df, on=['repo_name', 'dependency_name', 'version'])
merged = deps.merge(bundled_df[pd.notnull(bundled_df.is_bundled)], how='left', on=['repo_name', 'dependency_name', 'version'])
merged.fillna(False, inplace=True)
merged = merged.drop_duplicates()

bundled_projects = merged[merged['is_bundled'] == True]
bundled_projects = bundled_projects.groupby(['repo_name']).size().to_frame('count').reset_index()

"""unbundled_projects = merged[~merged['repo_name'].isin(bundled_projects['repo_name'])]

dep_count_per_project = unbundled_projects.groupby(['repo_name']).size().to_frame('count').reset_index()
## not per project, but per "dataset"
avg_unbundled_projects_depth = unbundled_projects['depth'].mean()
median_unbundled_projects_depth = unbundled_projects['depth'].median()
min_unbundled_projects_depth = unbundled_projects['depth'].min()
max_unbundled_projects_depth = unbundled_projects['depth'].max()"""

### Projects descriptive stats ###
# %%
df = pd.read_csv('data/dataset.csv')
min_stars = df['stars'].min()
max_stars = df['stars'].max()
avg_stars = df['stars'].mean()
median_stars = df['stars'].median

### RQ1 - Analysis on projects that ship something ###
# %%

# Cross reference all deps with bundled deps
dps_df = pd.read_csv('../data/npm_ls_dependencies.csv')
dps_df.loc[:, 'root'] = dps_df.path.map(lambda x: x.split("'")[1])
dps_df = dps_df.groupby(["repo_name", "dependency_name", "version"]).agg(
    {"depth": "min", "path": "count", "root": "first", "is_dev": "first", "is_runtime": "first"}).reset_index()

bundled_df = pd.read_csv('../data/bundled_deps_version.csv')

bundled_deps = dps_df.merge(bundled_df[pd.notnull(bundled_df.is_bundled)], how='left', on=['repo_name', 'dependency_name', 'version'])
bundled_deps.fillna(False, inplace=True)
bundled_deps = bundled_deps.drop_duplicates()

# Projects that ship something, deps grouped by version and repos
projects_ship = bundled_deps[bundled_deps["repo_name"].isin(bundled_df["repo_name"].unique())]
projects_ship = projects_ship.groupby(["repo_name", "dependency_name", "version"]).agg(
    {"depth": "min", "path": "count", "root": "first", "is_dev": "first", "is_runtime": "first", "is_bundled": "first"}).reset_index()

#deps grouped by version only
unique_deps = bundled_df.groupby(['dependency_name', 'version', 'repo_name'])

#how many times is a dep bundled and not bundled

bundled_deps['bundled_count'] = bundled_deps.groupby(['dependency_name', 'version'])['version'].transform(len)

"""# How many projects have bundled, dev dependencies? 35 out of 48
projects_ship_bundled_dev = projects_ship[(projects_ship['is_bundled'] == True) & (projects_ship['is_dev'] == True)]
nb_projects_bundled_dev = projects_ship_bundled_dev.groupby(['repo_name']).size().to_frame('count').reset_index()

# What is the most common bundle, dev dependency? 
most_common_bundled_dev = projects_ship_bundled_dev['root'].value_counts()[:].sort_values(ascending=False)

# Nb of dev deps, runtime, direct, and transitive
projects_ship_nb_dev_deps = len(projects_ship[projects_ship['is_dev'] == True])
projects_ship_nb_runtime_deps = len(projects_ship[projects_ship['is_runtime'] == True])
projects_ship_nb_direct_deps = len(projects_ship[projects_ship['depth'] == 1])
projects_ship_nb_transitive_deps = len(projects_ship[projects_ship['depth'] > 1])

# Nb direct, dev deps and nb runtime, direct deps
projects_ship_dev_direct = len(projects_ship[(projects_ship['depth'] == 1) & (projects_ship['is_dev'] == True)])
projects_ship_runtime_direct = len(projects_ship[(projects_ship['depth'] == 1) & (projects_ship['is_runtime'] == True)])

# Nb transitive runtime and transitive dev
projects_ship_runtime_transitive = len(projects_ship[(projects_ship['depth'] > 1) & (projects_ship['is_runtime'] == True)])
projects_ship_dev_transitive = len(projects_ship[(projects_ship['depth'] > 1) & (projects_ship['is_dev'] == True)])

# Analysis on the depth for projects that ship something
avg_depth_bundled = projects_ship['depth'].mean()
median_depth_bundled = projects_ship['depth'].median()
min_depth_bundled = projects_ship['depth'].min()
max_depth_bundled = projects_ship['depth'].max()

# Nb bundled deps
projects_ship_nb_bundled_deps = len(projects_ship[projects_ship['is_bundled'] == True])

# Nb bundled, dev - bundled,dev,transitive, bundled,dev,direct
projects_ship_bundled_dev = len(projects_ship[(projects_ship['is_bundled'] == True) & (projects_ship['is_dev'] == True)])
projects_ship_bundled_dev_transitive = len(projects_ship[(projects_ship['is_bundled'] == True) & (projects_ship['is_dev'] == True) & (projects_ship['depth'] > 1)])
projects_ship_bundled_dev_direct = len(projects_ship[(projects_ship['is_bundled'] == True) & (projects_ship['is_dev'] == True) & (projects_ship['depth'] == 1)])

# Nb bundled,runtime - bundled,runtime,transitive - bundled,runtime,direct
projects_ship_bundled_runtime = len(projects_ship[(projects_ship['is_bundled'] == True) & (projects_ship['is_runtime'] == True)])
projects_ship_bundled_runtime_transitive = len(projects_ship[(projects_ship['is_bundled'] == True) & (projects_ship['is_runtime'] == True) & (projects_ship['depth'] > 1)])
projects_ship_bundled_runtime_direct = len(projects_ship[(projects_ship['is_bundled'] == True) & (projects_ship['is_runtime'] == True) & (projects_ship['depth'] == 1)])

# Projects that ship nothing
projects_not_ship = bundled_deps[~bundled_deps["repo_name"].isin(bundled_df["repo_name"].unique())]
projects_not_ship = projects_not_ship.groupby(["repo_name", "dependency_name", "version"]).agg(
    {"depth": "min", "path": "count", "root": "first", "is_dev": "first", "is_runtime": "first", "is_bundled": "first"}).reset_index()

# Nb of dev deps 
projects_not_ship_nb_dev = len(projects_not_ship[projects_not_ship['is_dev'] == True])

# Nb runtime deps
projects_not_ship_nb_runtime = len(projects_not_ship[projects_not_ship['is_runtime'] == True])

# Analysis on the depth for projects that ship nothing
avg_depth_not_bundled = projects_not_ship['depth'].mean()
median_depth_not_bundled = projects_not_ship['depth'].median()
min_depth_not_bundled = projects_not_ship['depth'].min()
max_depth_not_bundled = projects_not_ship['depth'].max()"""

# Missing peer deps - uncategorized dependencies
# %%
dps = pd.read_csv('../data/npm_ls_dependencies2.csv')
dps.loc[:, 'root'] = dps.path.map(lambda x: x.split("'")[1])
dps_df = dps.groupby(["repo_name", "dependency_name", "version"]).agg(
    {"path": "first", 'root': 'first'}).reset_index()

bundled_df = pd.read_csv('../data/bundled_deps_version.csv')

bundled_deps = dps_df.merge(bundled_df[pd.notnull(bundled_df.is_bundled)], how='left', on=['repo_name', 'dependency_name', 'version'])
bundled_deps.fillna(False, inplace=True)
bundled_deps = bundled_deps.drop_duplicates()

in_tree = bundled_df.merge(bundled_deps, on=['repo_name', 'dependency_name', 'version'])

not_in_tree = bundled_df.merge(in_tree, how='outer', indicator=True).loc[lambda x: x['_merge'] == 'left_only']

missing_version = dps[dps['version'].isna()].groupby(['repo_name']).size().to_frame('count').reset_index()

#### RQ2 ####
# npm audit
# %%
deps = pd.read_csv('../data/npm_ls_dependencies.csv')
deps = deps.groupby(["repo_name", "dependency_name", "version"]).agg(
        {"depth": "min", "path": "count", "is_runtime": "first", "is_dev": "first"}).reset_index()

bundled_df = pd.read_csv('../data/bundled_deps_version.csv')

merged = deps.merge(bundled_df, on=['repo_name', 'dependency_name', 'version'])
merged = deps.merge(bundled_df[pd.notnull(bundled_df.is_bundled)], how='left', on=['repo_name', 'dependency_name', 'version'])
merged.fillna(False, inplace=True)
merged = merged.drop_duplicates()

audits = pd.read_csv('../data/npm_audit_attack_types.csv')
audits = audits.drop_duplicates()

reports = audits.merge(bundled_df[pd.notnull(bundled_df.is_bundled)], how='left', on=['repo_name', 'dependency_name', 'version'])
reports.fillna(False, inplace=True)

vuln_deps = reports.groupby(["repo_name", "dependency_name", "version"]).agg({"info":"count", "severity": "first", "title": "first", "is_bundled": "first", "vuln_type": "first"}).reset_index()

# How many vulnerable deps bundled?
bundled_vuln = vuln_deps[vuln_deps['is_bundled'] == True]

# Characteristics of vulnerable deps
"""vuln_dev = vuln_deps[vuln_deps['is_dev'] == True]
vuln_runtime = vuln_deps[vuln_deps['is_runtime'] == True]
vuln_transitive = vuln_deps[vuln_deps['depth'] > 1]
vuln_direct = vuln_deps[vuln_deps['depth'] == 1]

vuln_dev_direct = vuln_deps[(vuln_deps['is_dev'] == True) & (vuln_deps['depth'] == 1)]
vuln_dev_transitive = vuln_deps[(vuln_deps['is_dev'] == True) & (vuln_deps['depth'] > 1)]

vuln_runtime_direct = vuln_deps[(vuln_deps['is_runtime'] == True) & (vuln_deps['depth'] == 1)]
vuln_runtime_transitive = vuln_deps[(vuln_deps['is_runtime'] == True) & (vuln_deps['depth'] > 1)]"""

# Characteristics of security reports

vuln_types = reports['vuln_type'].value_counts()
severity = reports[reports['severity'] == 'high']['vuln_type'].value_counts()

# Vulnerable dependencies + attack types
# %%
deps = pd.read_csv('../data/npm_ls_dependencies.csv')
deps = deps.groupby(["repo_name", "dependency_name", "version"]).agg(
        {"depth": "min", "path": "count", "is_runtime": "first", "is_dev": "first"}).reset_index()

bundled_df = pd.read_csv('../data/bundled_deps_version.csv')

merged = deps.merge(bundled_df, on=['repo_name', 'dependency_name', 'version'])
merged = deps.merge(bundled_df[pd.notnull(bundled_df.is_bundled)], how='left', on=['repo_name', 'dependency_name', 'version'])
merged.fillna(False, inplace=True)
merged = merged.drop_duplicates()

audits = pd.read_csv('../data/npm_audit_attack_types.csv')
audits = audits.drop_duplicates()

reports = audits.merge(bundled_df[pd.notnull(bundled_df.is_bundled)], how='left', on=['repo_name', 'dependency_name', 'version'])
reports.fillna(False, inplace=True)

reports_deps = reports.merge(merged, on=['repo_name', 'dependency_name', 'version'], how='left')

"""redos_direct_dev = reports_deps[(reports_deps['vuln_type'] == 'Regular Expression Denial of Service') & (reports_deps['depth_y'] == 1) & (reports_deps['is_dev'] == True)]
redos_direct_runtime = reports_deps[(reports_deps['vuln_type'] == 'Regular Expression Denial of Service') & (reports_deps['depth_y'] == 1) & (reports_deps['is_runtime'] == True)]

redos_trans_dev = reports_deps[(reports_deps['vuln_type'] == 'Regular Expression Denial of Service') & (reports_deps['depth_y'] > 1) & (reports_deps['is_dev'] == True)]
redos_trans_runtime = reports_deps[(reports_deps['vuln_type'] == 'Regular Expression Denial of Service') & (reports_deps['depth_y'] > 1) & (reports_deps['is_runtime'] == True)]

proto_direct_dev = reports_deps[(reports_deps['vuln_type'] == 'Prototype Pollution') & (reports_deps['depth_y'] == 1) & (reports_deps['is_dev'] == True)]
proto_direct_runtime = reports_deps[(reports_deps['vuln_type'] == 'Prototype Pollution') & (reports_deps['depth_y'] == 1) & (reports_deps['is_runtime'] == True)]

proto_trans_dev = reports_deps[(reports_deps['vuln_type'] == 'Prototype Pollution') & (reports_deps['depth_y'] > 1) & (reports_deps['is_dev'] == True)]
proto_trans_runtime = reports_deps[(reports_deps['vuln_type'] == 'Prototype Pollution') & (reports_deps['depth_y'] > 1) & (reports_deps['is_runtime'] == True)]
"""

# %%
