# %%
import pandas as pd
import json
import csv

"""
Approach 1: All deps declared in devDependencies
    - No runtime dependencies
Approach 2: All deps declared in dependencies
    - No dev dependencies
Approach 3: Node convention: deps split into devDependencies and dependencies

None just means no dependencies declared at all, which should not happen in the context of our study
"""

def determine_approach(packagejson): 
    contents = json.load(packagejson)
    if 'devDependencies' in contents and 'dependencies' not in contents:
        if len(contents['devDependencies']) != 0:
            return 'devDependencies'
        if len(contents['devDependencies']) == 0:
            return 'None'
    
    if 'devDependencies' in contents and 'dependencies' in contents:
        if len(contents['devDependencies']) != 0 and len(contents['dependencies']) == 0:
            return 'devDependencies'
        if len(contents['devDependencies']) == 0 and len(contents['dependencies']) == 0:
            'None'
        if len(contents['devDependencies']) == 0 and len(contents['dependencies']) != 0:
            return 'dependencies'
        if len(contents['devDependencies']) != 0 and len(contents['dependencies']) != 0:
            return 'conventional'
        
    if 'devDependencies' not in contents and 'dependencies' in contents:
        if len(contents['dependencies']) == 0:
            return 'None'
        else:
            return 'dependencies'
    else:
        return 'None'

dataset = pd.read_csv('../data/dataset.csv')
repos = dataset['repo_path'].unique()
file_out = open('../data/projects_dep_configs.csv', 'w')
csv_out = csv.DictWriter(file_out, ['repo_name', 'dep_config'])
csv_out.writeheader()

for repo in repos:
    repo_name = str(repo).split('/')[-1]
    with open(repo + '/package.json', 'r') as f:
        approach = determine_approach(f)
        csv_out.writerow({'repo_name': repo_name, 'dep_config': approach})
file_out.close()

"""obj = {'devDependencies': {'dep1': 'v1'}, 'dependencies': {}}
json_obj = json.dumps(obj)
print(determine_approach(json_obj))
"""

# %%
import pandas as pd

deps = pd.read_csv('../data/npm_ls_dependencies.csv')

bundled_df = pd.read_csv('../data/bundled_deps_version.csv')

merged = deps.merge(bundled_df, on=['repo_name', 'dependency_name', 'version'])
merged = deps.merge(bundled_df[pd.notnull(bundled_df.is_bundled)], how='left', on=['repo_name', 'dependency_name', 'version'])
merged.fillna(False, inplace=True)
merged = merged.drop_duplicates()

#Filter all direct dependencies
direct_deps = merged[merged['depth'] == 1]

#For every package A on the list (not package + version, for each package name), find out:
#How often package A is installed AND released to production
#How often package A is installed AND not released to production

total_installed = direct_deps.groupby('dependency_name').size().to_frame('total_installed').reset_index()

direct_deps = pd.merge(direct_deps, total_installed, on='dependency_name')
direct_deps['installed_prod'] = direct_deps.apply(lambda x: direct_deps[(direct_deps['dependency_name'] == x['dependency_name']) & (direct_deps['is_bundled'] == True)].shape[0], axis=1)

direct_deps['installed_notprod'] = direct_deps.apply(lambda x: direct_deps[(direct_deps['dependency_name'] == x['dependency_name']) & (direct_deps['is_bundled'] == False)].shape[0], axis=1)

unique_deps = direct_deps.groupby('dependency_name').agg({'dependency_name': 'first', 'installed_prod': 'first', 'installed_notprod': 'first', 'total_installed': 'first'})

unique_deps.to_csv('../data/unique_deps_rq2.csv', mode='w', index=False)

#Always in prod
always_prod = unique_deps[(unique_deps['installed_prod'] != 0) & (unique_deps['installed_notprod'] == 0)]

#Never in prod
never_prod = unique_deps[(unique_deps['installed_notprod'] != 0) & (unique_deps['installed_prod'] == 0)]

#Sometimes in prod, sometimes not in prod
in_between = unique_deps[(unique_deps['installed_notprod'] != 0) & (unique_deps['installed_prod'] != 0)]

#At least once in prod
prod = unique_deps[(unique_deps['installed_prod'] != 0)]
# %%
